"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var value_type_enum_1 = require("./enums/value-type.enum");
var utils_1 = require("./utils");
var Compiler = /** @class */ (function () {
    function Compiler() {
    }
    Compiler.compile = function (tree) {
        var schema = {
            '$id': 'http://example.com/example.json',
            '$schema': 'http://json-schema.org/draft-07/schema#',
            type: tree.type,
            definitions: {},
            description: 'root of schema'
        };
        if (tree.type === value_type_enum_1.ValueTypeEnum.OBJECT) {
            schema = __assign({}, schema, { properties: {}, required: [] });
            Compiler.compileChild(tree, schema.properties, schema);
        }
        else {
            schema = __assign({}, schema, { uniqueItems: tree.uniqueItems, items: [] });
            Compiler.compileChild(tree, schema.items, schema);
            if (Object.keys(schema.items).length === 1) {
                schema.items = schema.items[0];
            }
        }
        return schema;
    };
    Compiler.compileChild = function (tree, properties, parentSchema) {
        var keys = Object.keys(tree.children);
        keys.forEach(function (k) {
            var child = tree.children[k];
            if (child.required && parentSchema.required) {
                parentSchema.required.push(k);
            }
            if (child.type === value_type_enum_1.ValueTypeEnum.OBJECT) {
                properties[k] = Compiler.getObjectPart(Compiler.getId(parentSchema, k, keys.length), child);
                Compiler.compileChild(child, properties[k].properties, properties[k]);
            }
            else if (child.type === value_type_enum_1.ValueTypeEnum.ARRAY) {
                properties[k] = Compiler.getArrayPart(Compiler.getId(parentSchema, k, keys.length), child);
                Compiler.compileChild(child, properties[k].items, properties[k]);
                if (Object.keys(properties[k].items).length === 1) {
                    properties[k].items = properties[k].items[0];
                }
            }
            else {
                properties[k] = Compiler.getPimitivePart(Compiler.getId(parentSchema, k, keys.length), child, k);
            }
        });
    };
    Compiler.getPimitivePart = function (id, child, k) {
        // TODO : Manage all options
        var result = {
            '$id': id,
            type: child.type,
            title: "The " + k + " Schema ",
            default: utils_1.Utils.getDefaultValue(child.type)
        };
        if (child.values && child.values.length > 0) {
            result.examples = child.values.slice();
        }
        return result;
    };
    Compiler.getObjectPart = function (id, child) {
        var schema = {
            '$id': id,
            type: child.type
        };
        if (Object.keys(child.children).length > 0) {
            schema = __assign({}, schema, { properties: {}, required: [] });
        }
        return schema;
    };
    Compiler.getArrayPart = function (id, child) {
        return {
            '$id': id,
            type: child.type,
            uniqueItems: child.uniqueItems,
            items: []
        };
    };
    Compiler.getId = function (parentSchema, key, length) {
        var parentId = parentSchema.$id[0] === '/' ? parentSchema.$id : '';
        if (parentSchema.type === value_type_enum_1.ValueTypeEnum.ARRAY) {
            if (length > 1) {
                return parentId + "/items/" + key;
            }
            else {
                return parentId + "/items";
            }
        }
        else {
            return parentId + "/properties/" + key;
        }
    };
    return Compiler;
}());
exports.Compiler = Compiler;
